//------------------------------------------------------------------------------
// multigenHD_romgen.v
//
// Block RAM init file generation utility for multigenHD
//
//
//
//                  Author: John F. Snow
//                  Staff Applications Engineer
//
//                  Video Applications
//                  Advanced Products Group
//                  Xilinx, Inc.
//
//                  Copyright (c) 2004 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Mar 2, 2004
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision:
//          Mar   2, 2004     XAPP682 Version 1.0 release
//
// Other modules instanced in this design:
//
//          None.
//------------------------------------------------------------------------------
/*
This module will generate nine output files, three for each of the three ROMs
used in multigenHD: the horizontal ROM, the vertical ROM, and the color ROM. 
One file contains the simulation initialization code, one contains the XST 
synthesis attributes and the third contains the Synplicity synthesis attributes.
Note, this module is not synthesizable. It is intended to be run using ModelSim
to create the initialization files for block RAMs used in the multigenHD module.

*/

`timescale 1ns / 1ns

module multirateHD_romgen ();

//-----------------------------------------------------------------------------
// Parameter definitions
//

//
// This group of parameters defines the bit widths of various fields in the
// module. Note that if the VID_WIDTH parameter is changed, the video component
// values for the various colors will need to be modified accordingly.
//
parameter VID_WIDTH     = 10;                   // Width of video components
parameter HCNT_WIDTH    = 12;                   // Width of h_counter
parameter VCNT_WIDTH    = 11;                   // Width of v_counter
parameter HRGN_WIDTH    = 5;                    // Width of h_region counter
parameter VRGN_WIDTH    = 5;                    // Width of v_region counter
parameter VBAND_WIDTH   = 3;                    // Width of vband code
parameter YRAMP_FRAC    = 7;                    // Number of fractional bits in Y Ramp reg
parameter YRAMP_WIDTH   = VID_WIDTH + YRAMP_FRAC;// Width of Y Ramp reg

parameter V_EVNT_WIDTH  = VCNT_WIDTH;           // Width of v_next_evnt
parameter H_EVNT_WIDTH  = HCNT_WIDTH - 1;       // Width of h_next_evnt
 
parameter VID_MSB       = VID_WIDTH - 1;        // MS bit # of video data path
parameter HCNT_MSB      = HCNT_WIDTH - 1;       // MS bit # of h_counter
parameter VCNT_MSB      = VCNT_WIDTH - 1;       // MS bit # of v_counter
parameter HRGN_MSB      = HRGN_WIDTH - 1;       // MS bit # of h_region counter
parameter VRGN_MSB      = VRGN_WIDTH - 1;       // MS bit # of v_region counter
parameter VBAND_MSB     = VBAND_WIDTH - 1;      // MS bit # of vband code
parameter V_EVNT_MSB    = V_EVNT_WIDTH - 1;     // MS bit # of v_next_evnt
parameter H_EVNT_MSB    = H_EVNT_WIDTH - 1;     // MS bit # of h_next_evnt
parameter YRAMP_MSB     = YRAMP_WIDTH - 1;      // MS bit # of y_ramp register

//
// This group of parameters defines the horizontal regions from the horizontal
// ROM. Note that the three HRGN_USROPTx regions are never generated by the
// horizontal ROM. An encoder will change HRGN_BAR1 to any of these three values
// depending on the user option inputs before feeding the modified horizontal
// region code to the color ROM. The encoder will also determine whether to
// output the HRGN_CEQ_POL0 or POL1 code depending on whether the current frame
// is odd or even.
//
parameter [HRGN_MSB:0]
    HRGN_BAR0           = 0,    // 40% gray
    HRGN_BAR1           = 1,    // 75% white (left part 0% black)
    HRGN_BAR2           = 2,    // left part 75% yellow (right part 0% black)
    HRGN_BAR3           = 3,    // right part 75% yellow (right part 100% white)
    HRGN_BAR4           = 4,    // 75% cyan (middle part 100% white)
    HRGN_BAR5           = 5,    // left part 75% green (left part 100% white)
    HRGN_BAR6           = 6,    // right part 75% green (left part 0% black)
    HRGN_BAR7           = 7,    // left part 75% magenta (right part 0% black) 
    HRGN_BAR8           = 8,    // middle part 75% magenta (-2% black)
    HRGN_BAR9           = 9,    // right part 75% magenta (0% black)
    HRGN_BAR10          = 10,   // left part 75% red (+2% black)
    HRGN_BAR11          = 11,   // middle part 75% red (0% black)
    HRGN_BAR12          = 12,   // right part 75% red (+4% black)
    HRGN_BAR13          = 13,   // 75% blue (0% black)
    HRGN_BAR14          = 14,   // 40% gray
    HRGN_INCV           = 15,   // increment vertical line number
    HRGN_EAV1           = 16,   // first two words of EAV
    HRGN_EAV2_F0        = 17,   // second two words of EAV for field 0
    HRGN_HBLANK         = 18,   // horizontal blanking period
    HRGN_SAV1           = 19,   // first two words of SAV
    HRGN_SAV2_F0        = 20,   // second two words of SAV for field 0
    HRGN_LN             = 21,   // two words of line number
    HRGN_CRC            = 22,   // two words of CRC
    HRGN_SAV2_F1        = 23,   // second two words of SAV for field 1
    HRGN_USROPT1        = 24,   // BAR1 but with 100% white in pat 2, 0% black in pat 3
    HRGN_USROPT2        = 25,   // BAR1 but with +I in pat 2, 0% black in pat 3
    HRGN_USROPT3        = 26,   // BAR1 but with -I in pat 2, +Q in pat 3
    HRGN_EAV2_F1        = 27,   // second two words of EAV for field 1
    HRGN_CEQ_PLL        = 28,   // RP 198 cable equalization pattern
    HRGN_CEQ_POL_0      = 29,   // RP 198 cable eq pattern, polarity word even frame
    HRGN_CEQ_POL_1      = 30,   // RP 198 cable eq pattern, polarity word odd frame
    HRGN_RST            = 31;   // initial state after reset


//
// The group of parameters defines the vertical regions from the vertical ROM.
// Note that the vertical ROM generates a 5-bit vertical region code and a 3-bit
// vertical band code. The region code is essentially the current state of the
// vertical state machine and feeds back to the input of the vertical ROM.
// The vertical band code is sent to the color ROM to indicate the current
// vertical pattern band.
//
parameter [VRGN_MSB:0]
    VRGN_FM0_F0_VB_0    = 0,    // frame 0, field 0, first vertical blanking interval
    VRGN_FM0_F0_1ST_ACT = 1,    // frame 0, field 0, first active line (for CEQ polarity)
    VRGN_FM0_F0_PAT1    = 2,    // frame 0, field 0, pattern 1 or cable eq pattern
    VRGN_FM0_F0_PAT2    = 3,    // frame 0, field 0, pattern 2 or PLL pattern
    VRGN_FM0_F0_PAT3    = 4,    // frame 0, field 0, pattern 3
    VRGN_FM0_F0_PAT4    = 5,    // frame 0, field 0, pattern 4
    VRGN_FM0_F0_VB_1    = 6,    // frame 0, field 0, second vertical blanking interval
    VRGN_FM0_F1_VB_0    = 7,    // frame 0, field 1, first vertical blanking interval
    VRGN_FM0_F1_PAT1    = 8,    // frame 0, field 1, pattern 1 or cable eq pattern
    VRGN_FM0_F1_PAT2    = 9,    // frame 0, field 1, pattern 2 or PLL pattern
    VRGN_FM0_F1_PAT3    = 10,   // frame 0, field 1, pattern 3
    VRGN_FM0_F1_PAT4    = 11,   // frame 0, field 1, pattern 4
    VRGN_FM0_F1_VB_1    = 12,   // frame 0, field 1, second vertical blanking interval
    VRGN_FM0_CLRV       = 13,   // frame 0 clears the vertical counter back to 1
    VRGN_X14            = 14,   // unused
    VRGN_X15            = 15,   // unused
    VRGN_FM1_F0_VB_0    = 16,   // frame 1, field 0, first vertical blanking interval
    VRGN_FM1_F0_PAT1    = 17,   // frame 1, field 0, pattern 1 or cable eq pattern
    VRGN_FM1_F0_PAT2    = 18,   // frame 1, field 0, pattern 2 or PLL pattern
    VRGN_FM1_F0_PAT3    = 19,   // frame 1, field 0, pattern 3
    VRGN_FM1_F0_PAT4    = 20,   // frame 1, field 0, pattern 4
    VRGN_FM1_F0_VB_1    = 21,   // frame 1, field 0, second vertical blanking interval
    VRGN_X22            = 22,   // unused
    VRGN_FM1_F1_VB_0    = 23,   // frame 1, field 1, first vertical blanking interval
    VRGN_FM1_F1_PAT1    = 24,   // frame 1, field 1, pattern 1 or cable eq pattern
    VRGN_FM1_F1_PAT2    = 25,   // frame 1, field 1, pattern 2 or PLL pattern
    VRGN_FM1_F1_PAT3    = 26,   // frame 1, field 1, pattern 3
    VRGN_FM1_F1_PAT4    = 27,   // frame 1, field 1, pattern 4
    VRGN_FM1_F1_VB_1    = 28,   // frame 1, field 1, second vertical blanking interval
    VRGN_FM1_CLRV       = 29,   // frame 1 clears the vertical counter back to 1
    VRGN_X30            = 30,   // unused
    VRGN_RST            = 31;   // initial state after reset

parameter [VBAND_MSB:0]
    VBAND_VB            = 0,    // vertical blanking interval
    VBAND_PAT1          = 1,    // pattern 1
    VBAND_PAT2          = 2,    // pattern 2
    VBAND_PAT3          = 3,    // pattern 3
    VBAND_PAT4          = 4,    // pattern 4
    VBAND_CEQ           = 5,    // cable equalization pattern
    VBAND_PLL           = 6,    // PLL pattern
    VBAND_X7            = 7;    // unused

//
// Video standard parameters
//
// These codes are not the same as the 3-bit std code that tells the ROMs which
// standard to display. These are 4-bit codes used to specify a larger set of
// standards that can mapped into the eight available codes that can be 
// implemented in the 3-block RAM configuration. The function convert_std
// is used to map these 4-bit codes to the 3-bit std codes. This function must
// be customized to specify which codes are included in the supported set.
//
parameter [3:0]
    VSTD_260M           = 4'b0000,      // SMPTE 260M, 1035i 30 Hz & 29.97Hz
    VSTD_295M           = 4'b0001,      // SMPTE 295M, 1080i 25Hz
    VSTD_274M_30i       = 4'b0010,      // SMPTE 274M, 1080i 30 Hz & 29.97Hz (also RP211 1080sF 30Hz & 29.97Hz)
    VSTD_274M_25i       = 4'b0011,      // SMPTE 274M, 1080i 25 Hz (also RP211 1080sF 25Hz)
    VSTD_274M_30p       = 4'b0100,      // SMPTE 274M, 1080p 30Hz & 29.97Hz
    VSTD_274M_25p       = 4'b0101,      // SMPTE 274M, 1080p 25Hz
    VSTD_274M_24p       = 4'b0110,      // SMPTE 274M, 1080p 24Hz and 23.98Hz
    VSTD_296M           = 4'b0111,      // SMPTE 296M, 720p 60 Hz and 59.94Hz
    VSTD_274M_24sF      = 4'b1000,      // SMPTE 274M (RP211) 1080sF 24 Hz and 23.98Hz
    VSTD_UNDEFINED_9    = 4'b1001,
    VSTD_UNDEFINED_A    = 4'b1010,
    VSTD_UNDEFINED_B    = 4'b1011,
    VSTD_UNDEFINED_C    = 4'b1100,
    VSTD_UNDEFINED_D    = 4'b1101,
    VSTD_UNDEFINED_E    = 4'b1110,
    VSTD_UNDEFINED_F    = 4'b1111; 

//
// Component video values for various parts of the pattern. These values are 
// taken, for the most part, from the color values specified in RP 219-2002.
//
parameter [VID_MSB:0]
    Y_WHITE75   = 10'd721,  CB_WHITE75   = 10'd512,     CR_WHITE75   = 10'd512,     // 75% White
    Y_YELLOW75  = 10'd674,  CB_YELLOW75  = 10'd176,     CR_YELLOW75  = 10'd543,     // 75% Yellow
    Y_CYAN75    = 10'd581,  CB_CYAN75    = 10'd589,     CR_CYAN75    = 10'd176,     // 75% Cyan
    Y_GREEN75   = 10'd534,  CB_GREEN75   = 10'd253,     CR_GREEN75   = 10'd207,     // 75% Green
    Y_MAGENTA75 = 10'd251,  CB_MAGENTA75 = 10'd771,     CR_MAGENTA75 = 10'd817,     // 75% Magenta
    Y_RED75     = 10'd204,  CB_RED75     = 10'd435,     CR_RED75     = 10'd848,     // 75% Red
    Y_BLUE75    = 10'd111,  CB_BLUE75    = 10'd848,     CR_BLUE75    = 10'd481,     // 75% Blue
    Y_GRAY40    = 10'd414,  CB_GRAY40    = 10'd512,     CR_GRAY40    = 10'd512,     // 40% Gray
    Y_CYAN100   = 10'd754,  CB_CYAN100   = 10'd615,     CR_CYAN100   = 10'd64,      // 100% Cyan
    Y_WHITE100  = 10'd940,  CB_WHITE100  = 10'd512,     CR_WHITE100  = 10'd512,     // 100% White
    Y_BLUE100   = 10'd127,  CB_BLUE100   = 10'd960,     CR_BLUE100   = 10'd471,     // 100% Blue
    Y_PLUS_I    = 10'd245,  CB_PLUS_I    = 10'd412,     CR_PLUS_I    = 10'd629,     // +I
    Y_MINUS_I   = 10'd244,  CB_MINUS_I   = 10'd612,     CR_MINUS_I   = 10'd395,     // -I
    Y_YELLOW100 = 10'd877,  CB_YELLOW100 = 10'd64,      CR_YELLOW100 = 10'd553,     // 100% Yellow
    Y_RED100    = 10'd250,  CB_RED100    = 10'd409,     CR_RED100    = 10'd960,     // 100% Red
    Y_PLUS_Q    = 10'd141,  CB_PLUS_Q    = 10'd697,     CR_PLUS_Q    = 10'd606,     // +Q
    Y_GRAY15    = 10'd195,  CB_GRAY15    = 10'd512,     CR_GRAY15    = 10'd512,     // 15% Gray
    Y_BLK0      = 10'd64,   CB_BLK0      = 10'd512,     CR_BLK0      = 10'd512,     // 0% Black
    Y_BLK_M2    = 10'd46,   CB_BLK_M2    = 10'd512,     CR_BLK_M2    = 10'd512,     // -2% Black
    Y_BLK_P2    = 10'd82,   CB_BLK_P2    = 10'd512,     CR_BLK_P2    = 10'd512,     // 2% Black
    Y_BLK_P4    = 10'd99,   CB_BLK_P4    = 10'd512,     CR_BLK_P4    = 10'd512,     // 4% Black
    Y_BLANK     = 10'd64,   CB_BLANK     = 10'd512,     CR_BLANK     = 10'd512,     // blanking value
    Y_CEQ       = 10'h198,  Y_CEQ_INV    = 10'h190,     C_CEQ        = 10'h300,     // RP198 cable EQ pattern
    Y_PLL       = 10'h110,  C_PLL        = 10'h200;                                 // RP198 PLL pattern

//-----------------------------------------------------------------------------
// Signal definitions
//

integer sim_file, xst_file, syn_file;                   // Verilog file handles
integer sim_file_vhdl, xst_file_vhdl, syn_file_vhdl;    // VHDL file handles
integer line, word;                                     // line & word counters for generating the INIT files
reg [6:0]   line_num;                                   // 7-bit line number
reg [2:0]   word_num;                                   // 3-bit word number

//-----------------------------------------------------------------------------
// Main code
//
// This code creates each ROM file by repeatedly calling each ROM function
// with each possible address choice and writing the returned value out to the
// corresponding ROM file as an INIT statement.
//

initial
    begin

        //
        // Create the HROM initialization files
        //

        // Open the six HROM output files.
        sim_file = $fopen("hrom_sim.v");
        xst_file = $fopen("hrom_xst.v");
        syn_file = $fopen("hrom_syn.v");
    
        sim_file_vhdl = $fopen("hrom_sim.vhd");
        xst_file_vhdl = $fopen("hrom_xst.vhd");
        syn_file_vhdl = $fopen("hrom_syn.vhd");

        // Write the header information for each file including the INIT_P
        // attributes. The parity bits are not used by the HROM.
        $fdisplay(sim_file, "// Simulation initialization code HROM");
        $fdisplay(sim_file, "// Created by multigenHD_romgen.v");
        $fdisplay(sim_file, "// Video format mapping:");
        $fdisplay(sim_file, "//   000 = %s", video_format_string(3'b000));
        $fdisplay(sim_file, "//   001 = %s", video_format_string(3'b001));
        $fdisplay(sim_file, "//   002 = %s", video_format_string(3'b010));
        $fdisplay(sim_file, "//   003 = %s", video_format_string(3'b011));
        $fdisplay(sim_file, "//   004 = %s", video_format_string(3'b100));
        $fdisplay(sim_file, "//   005 = %s", video_format_string(3'b101));
        $fdisplay(sim_file, "//   006 = %s", video_format_string(3'b110));
        $fdisplay(sim_file, "//   007 = %s", video_format_string(3'b111));
        $fdisplay(sim_file, "//synthesis translate_off");
        $fdisplay(sim_file, "defparam");
        $fdisplay(sim_file, "HROM.INIT = 36'h0%s,",  tohex8({9'b000000000, 1'b1, 1'b1, HRGN_BAR0, 11'd2047, HRGN_RST}));
        $fdisplay(sim_file, "HROM.SRVAL = 36'h0%s,", tohex8({9'b000000000, 1'b1, 1'b1, HRGN_BAR0, 11'd2047, HRGN_RST}));
        $fdisplay(sim_file, "HROM.WRITE_MODE = \"READ_FIRST\",");
        $fdisplay(sim_file, "HROM.INITP_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "HROM.INITP_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "HROM.INITP_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "HROM.INITP_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "HROM.INITP_04 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "HROM.INITP_05 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "HROM.INITP_06 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "HROM.INITP_07 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");

        $fdisplay(sim_file_vhdl, "-- Simulation initialization code HROM");
        $fdisplay(sim_file_vhdl, "-- Created by multigenHD_romgen.v");
        $fdisplay(sim_file_vhdl, "-- Video format mapping:");
        $fdisplay(sim_file_vhdl, "--   000 = %s", video_format_string(3'b000));
        $fdisplay(sim_file_vhdl, "--   001 = %s", video_format_string(3'b001));
        $fdisplay(sim_file_vhdl, "--   002 = %s", video_format_string(3'b010));
        $fdisplay(sim_file_vhdl, "--   003 = %s", video_format_string(3'b011));
        $fdisplay(sim_file_vhdl, "--   004 = %s", video_format_string(3'b100));
        $fdisplay(sim_file_vhdl, "--   005 = %s", video_format_string(3'b101));
        $fdisplay(sim_file_vhdl, "--   006 = %s", video_format_string(3'b110));
        $fdisplay(sim_file_vhdl, "--   007 = %s", video_format_string(3'b111));
        $fdisplay(sim_file_vhdl, "-- pragma translate_off");
        $fdisplay(sim_file_vhdl, "generic map (");
        $fdisplay(sim_file_vhdl, "INIT       => X\"%s\",", tohex8({9'b000000000, 1'b1, 1'b1, HRGN_BAR0, 11'd2047, HRGN_RST}));
        $fdisplay(sim_file_vhdl, "SRVAL      => X\"%s\",", tohex8({9'b000000000, 1'b1, 1'b1, HRGN_BAR0, 11'd2047, HRGN_RST}));
        $fdisplay(sim_file_vhdl, "WRITE_MODE => \"READ_FIRST\",");
        $fdisplay(sim_file_vhdl, "INITP_00 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_01 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_02 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_03 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_04 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_05 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_06 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_07 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");

        $fdisplay(xst_file, "// XST synthesis initialization code HROM");
        $fdisplay(xst_file, "// Created by multigenHD_romgen.v");
        $fdisplay(xst_file, "// Video format mapping:");
        $fdisplay(xst_file, "//   000 = %s", video_format_string(3'b000));
        $fdisplay(xst_file, "//   001 = %s", video_format_string(3'b001));
        $fdisplay(xst_file, "//   002 = %s", video_format_string(3'b010));
        $fdisplay(xst_file, "//   003 = %s", video_format_string(3'b011));
        $fdisplay(xst_file, "//   004 = %s", video_format_string(3'b100));
        $fdisplay(xst_file, "//   005 = %s", video_format_string(3'b101));
        $fdisplay(xst_file, "//   006 = %s", video_format_string(3'b110));
        $fdisplay(xst_file, "//   007 = %s", video_format_string(3'b111));
        $fdisplay(xst_file, "//synthesis attribute INIT of HROM is \"0%s\"",  tohex8({9'b000000000, 1'b1, 1'b1, HRGN_BAR0, 11'd2047, HRGN_RST}));
        $fdisplay(xst_file, "//synthesis attribute SRVAL of HROM is \"0%s\"", tohex8({9'b000000000, 1'b1, 1'b1, HRGN_BAR0, 11'd2047, HRGN_RST}));
        $fdisplay(xst_file, "//synthesis attribute WRITE_MODE of HROM is \"READ_FIRST\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_00 of HROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_01 of HROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_02 of HROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_03 of HROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_04 of HROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_05 of HROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_06 of HROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_07 of HROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        
        $fdisplay(xst_file_vhdl, "-- XST synthesis initialization code HROM");
        $fdisplay(xst_file_vhdl, "-- Created by multigenHD_romgen.v");
        $fdisplay(xst_file_vhdl, "-- Video format mapping:");
        $fdisplay(xst_file_vhdl, "--   000 = %s", video_format_string(3'b000));
        $fdisplay(xst_file_vhdl, "--   001 = %s", video_format_string(3'b001));
        $fdisplay(xst_file_vhdl, "--   002 = %s", video_format_string(3'b010));
        $fdisplay(xst_file_vhdl, "--   003 = %s", video_format_string(3'b011));
        $fdisplay(xst_file_vhdl, "--   004 = %s", video_format_string(3'b100));
        $fdisplay(xst_file_vhdl, "--   005 = %s", video_format_string(3'b101));
        $fdisplay(xst_file_vhdl, "--   006 = %s", video_format_string(3'b110));
        $fdisplay(xst_file_vhdl, "--   007 = %s", video_format_string(3'b111));
        $fdisplay(xst_file_vhdl, "attribute INIT of HROM : label is \"0%s\";",  tohex8({9'b000000000, 1'b1, 1'b1, HRGN_BAR0, 11'd2047, HRGN_RST}));
        $fdisplay(xst_file_vhdl, "attribute SRVAL of HROM : label is \"0%s\";", tohex8({9'b000000000, 1'b1, 1'b1, HRGN_BAR0, 11'd2047, HRGN_RST}));
        $fdisplay(xst_file_vhdl, "attribute WRITE_MODE of HROM : label is \"READ_FIRST\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_00 of HROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_01 of HROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_02 of HROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_03 of HROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_04 of HROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_05 of HROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_06 of HROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_07 of HROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");

        $fdisplay(syn_file, "/* synthesis xc_props=\"INIT=0%s,\\",  tohex8({9'b000000000, 1'b1, 1'b1, HRGN_BAR0, 11'd2047, HRGN_RST}));
        $fdisplay(syn_file, "SRVAL=0%s,\\", tohex8({9'b000000000, 1'b1, 1'b1, HRGN_BAR0, 11'd2047, HRGN_RST}));
        $fdisplay(syn_file, "WRITE_MODE=READ_FIRST,\\");
        $fdisplay(syn_file, "INITP_00=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_01=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_02=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_03=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_04=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_05=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_06=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_07=0000000000000000000000000000000000000000000000000000000000000000,\\");

        $fdisplay(syn_file_vhdl, "attribute xc_props of HROM : label is \"INIT=%s,\" &",  tohex8({9'b000000000, 1'b1, 1'b1, HRGN_BAR0, 11'd2047, HRGN_RST}));
        $fdisplay(syn_file_vhdl, "\"SRVAL=%s,\" &", tohex8({9'b000000000, 1'b1, 1'b1, HRGN_BAR0, 11'd2047, HRGN_RST}));
        $fdisplay(syn_file_vhdl, "\"WRITE_MODE=READ_FIRST,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_00=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_01=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_02=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_03=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_04=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_05=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_06=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_07=0000000000000000000000000000000000000000000000000000000000000000,\" &");

        // For each of the 64 INIT attributes, create a line with 256 bits 
        // encoded as hex and output to the various files.
        for (line = 0; line < 64; line = line + 1)
            begin
                line_num = line;
                $fwrite(sim_file,      "HROM.INIT_%s = 256'h", tohex2(line_num));
                $fwrite(sim_file_vhdl, "INIT_%s => X\"", tohex2(line_num));
                $fwrite(xst_file,      "//synthesis attribute INIT_%s of HROM is \"", tohex2(line_num));
                $fwrite(xst_file_vhdl, "attribute INIT_%s of HROM : label is \"", tohex2(line_num));
                $fwrite(syn_file,      "INIT_%s=", tohex2(line_num));
                $fwrite(syn_file_vhdl, "\"INIT_%s=", tohex2(line_num));

                for (word = 7; word >= 0; word = word - 1)
                    begin
                        word_num = word;
                        $fwrite(sim_file,      "%s", tohex8(hrom_value({line_num, word_num})));
                        $fwrite(sim_file_vhdl, "%s", tohex8(hrom_value({line_num, word_num})));
                        $fwrite(xst_file,      "%s", tohex8(hrom_value({line_num, word_num})));
                        $fwrite(xst_file_vhdl, "%s", tohex8(hrom_value({line_num, word_num})));
                        $fwrite(syn_file,      "%s", tohex8(hrom_value({line_num, word_num})));
                        $fwrite(syn_file_vhdl, "%s", tohex8(hrom_value({line_num, word_num})));
                    end
                if (line == 63)
                    begin
                        $fwrite(sim_file,      ";\n");
                        $fwrite(sim_file_vhdl, "\"\n");
                        $fwrite(xst_file,      "\"\n");
                        $fwrite(xst_file_vhdl, "\";\n");
                        $fwrite(syn_file,      "\"*/\n");
                        $fwrite(syn_file_vhdl, "\";\n");
                    end
                else
                    begin
                        $fwrite(sim_file,      ",\n");
                        $fwrite(sim_file_vhdl, "\",\n");
                        $fwrite(xst_file,      "\"\n");
                        $fwrite(xst_file_vhdl, "\";\n");
                        $fwrite(syn_file,      ",\\\n");
                        $fwrite(syn_file_vhdl, ",\" &\n");
                    end
            end

        // Now write any trailing lines to the files.
        $fdisplay(sim_file,"//synthesis translate_on");
        $fdisplay(sim_file_vhdl, ")");
        $fdisplay(sim_file_vhdl, "-- pragma translate_on");

        // Close the HROM files.
        $fclose(sim_file);
        $fclose(sim_file_vhdl);
        $fclose(xst_file);
        $fclose(xst_file_vhdl);
        $fclose(syn_file);
        $fclose(syn_file_vhdl);

        //
        // Create the VROM initialization files
        //

        // Open the six VROM output files
        sim_file = $fopen("vrom_sim.v");
        xst_file = $fopen("vrom_xst.v");
        syn_file = $fopen("vrom_syn.v");

        sim_file_vhdl = $fopen("vrom_sim.vhd");
        xst_file_vhdl = $fopen("vrom_xst.vhd");
        syn_file_vhdl = $fopen("vrom_syn.vhd");

        // Write the header information for each file including the INIT_P
        // attributes. The parity bits are not used by the VROM.

        $fdisplay(sim_file, "// Simulation initialization code VROM");
        $fdisplay(sim_file, "// Created by multigenHD_romgen.v");
        $fdisplay(sim_file, "// Video format mapping:");
        $fdisplay(sim_file, "//   000 = %s", video_format_string(3'b000));
        $fdisplay(sim_file, "//   001 = %s", video_format_string(3'b001));
        $fdisplay(sim_file, "//   002 = %s", video_format_string(3'b010));
        $fdisplay(sim_file, "//   003 = %s", video_format_string(3'b011));
        $fdisplay(sim_file, "//   004 = %s", video_format_string(3'b100));
        $fdisplay(sim_file, "//   005 = %s", video_format_string(3'b101));
        $fdisplay(sim_file, "//   006 = %s", video_format_string(3'b110));
        $fdisplay(sim_file, "//   007 = %s", video_format_string(3'b111));
        $fdisplay(sim_file, "//synthesis translate_off");
        $fdisplay(sim_file, "defparam");
        $fdisplay(sim_file, "VROM.INIT = 36'h0%s,",  tohex8({8'h00, 1'b1, 1'b0, 1'b0, 1'b1, VBAND_VB, 11'd2047, VRGN_RST}));
        $fdisplay(sim_file, "VROM.SRVAL = 36'h0%s,", tohex8({8'h00, 1'b1, 1'b0, 1'b0, 1'b1, VBAND_VB, 11'd2047, VRGN_RST}));
        $fdisplay(sim_file, "VROM.WRITE_MODE = \"READ_FIRST\",");
        $fdisplay(sim_file, "VROM.INITP_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "VROM.INITP_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "VROM.INITP_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "VROM.INITP_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "VROM.INITP_04 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "VROM.INITP_05 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "VROM.INITP_06 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "VROM.INITP_07 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");

        $fdisplay(sim_file_vhdl, "-- Simulation initialization code VROM");
        $fdisplay(sim_file_vhdl, "-- Created by multigenHD_romgen.v");
        $fdisplay(sim_file_vhdl, "-- Video format mapping:");
        $fdisplay(sim_file_vhdl, "--   000 = %s", video_format_string(3'b000));
        $fdisplay(sim_file_vhdl, "--   001 = %s", video_format_string(3'b001));
        $fdisplay(sim_file_vhdl, "--   002 = %s", video_format_string(3'b010));
        $fdisplay(sim_file_vhdl, "--   003 = %s", video_format_string(3'b011));
        $fdisplay(sim_file_vhdl, "--   004 = %s", video_format_string(3'b100));
        $fdisplay(sim_file_vhdl, "--   005 = %s", video_format_string(3'b101));
        $fdisplay(sim_file_vhdl, "--   006 = %s", video_format_string(3'b110));
        $fdisplay(sim_file_vhdl, "--   007 = %s", video_format_string(3'b111));
        $fdisplay(sim_file_vhdl, "-- pragma translate_off");
        $fdisplay(sim_file_vhdl, "generic map (");
        $fdisplay(sim_file_vhdl, "INIT       => X\"%s\",", tohex8({8'h00, 1'b1, 1'b0, 1'b0, 1'b1, VBAND_VB, 11'd2047, VRGN_RST}));
        $fdisplay(sim_file_vhdl, "SRVAL      => X\"%s\",", tohex8({8'h00, 1'b1, 1'b0, 1'b0, 1'b1, VBAND_VB, 11'd2047, VRGN_RST}));
        $fdisplay(sim_file_vhdl, "WRITE_MODE => \"READ_FIRST\",");
        $fdisplay(sim_file_vhdl, "INITP_00 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_01 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_02 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_03 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_04 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_05 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_06 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_07 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");

        $fdisplay(xst_file, "// XST synthesis initialization code VROM");
        $fdisplay(xst_file, "// Created by multigenHD_romgen.v");
        $fdisplay(xst_file, "// Video format mapping:");
        $fdisplay(xst_file, "//   000 = %s", video_format_string(3'b000));
        $fdisplay(xst_file, "//   001 = %s", video_format_string(3'b001));
        $fdisplay(xst_file, "//   002 = %s", video_format_string(3'b010));
        $fdisplay(xst_file, "//   003 = %s", video_format_string(3'b011));
        $fdisplay(xst_file, "//   004 = %s", video_format_string(3'b100));
        $fdisplay(xst_file, "//   005 = %s", video_format_string(3'b101));
        $fdisplay(xst_file, "//   006 = %s", video_format_string(3'b110));
        $fdisplay(xst_file, "//   007 = %s", video_format_string(3'b111));
        $fdisplay(xst_file, "//synthesis attribute INIT of VROM is \"0%s\"",   tohex8({8'h00, 1'b1, 1'b0, 1'b0, 1'b1, VBAND_VB, 11'd2047, VRGN_RST}));
        $fdisplay(xst_file, "//synthesis attribute SRVAL of VROM is \"0%s\"",  tohex8({8'h00, 1'b1, 1'b0, 1'b0, 1'b1, VBAND_VB, 11'd2047, VRGN_RST}));
        $fdisplay(xst_file, "//synthesis attribute WRITE_MODE of VROM is \"READ_FIRST\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_00 of VROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_01 of VROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_02 of VROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_03 of VROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_04 of VROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_05 of VROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_06 of VROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_07 of VROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");

        $fdisplay(xst_file_vhdl, "-- XST synthesis initialization code VROM");
        $fdisplay(xst_file_vhdl, "-- Created by multigenHD_romgen.v");
        $fdisplay(xst_file_vhdl, "-- Video format mapping:");
        $fdisplay(xst_file_vhdl, "--   000 = %s", video_format_string(3'b000));
        $fdisplay(xst_file_vhdl, "--   001 = %s", video_format_string(3'b001));
        $fdisplay(xst_file_vhdl, "--   002 = %s", video_format_string(3'b010));
        $fdisplay(xst_file_vhdl, "--   003 = %s", video_format_string(3'b011));
        $fdisplay(xst_file_vhdl, "--   004 = %s", video_format_string(3'b100));
        $fdisplay(xst_file_vhdl, "--   005 = %s", video_format_string(3'b101));
        $fdisplay(xst_file_vhdl, "--   006 = %s", video_format_string(3'b110));
        $fdisplay(xst_file_vhdl, "--   007 = %s", video_format_string(3'b111));
        $fdisplay(xst_file_vhdl, "attribute INIT of VROM : label is \"0%s\";",  tohex8({8'h00, 1'b1, 1'b0, 1'b0, 1'b1, VBAND_VB, 11'd2047, VRGN_RST}));
        $fdisplay(xst_file_vhdl, "attribute SRVAL of VROM : label is \"0%s\";", tohex8({8'h00, 1'b1, 1'b0, 1'b0, 1'b1, VBAND_VB, 11'd2047, VRGN_RST}));
        $fdisplay(xst_file_vhdl, "attribute WRITE_MODE of VROM : label is \"READ_FIRST\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_00 of VROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_01 of VROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_02 of VROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_03 of VROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_04 of VROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_05 of VROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_06 of VROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_07 of VROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");

        $fdisplay(syn_file, "/* synthesis xc_props=\"INIT=0%s,\\",  tohex8({8'h00, 1'b1, 1'b0, 1'b0, 1'b1, VBAND_VB, 11'd2047, VRGN_RST}));
        $fdisplay(syn_file, "SRVAL=0%s,\\", tohex8({8'h00, 1'b1, 1'b0, 1'b0, 1'b1, VBAND_VB, 11'd2047, VRGN_RST}));
        $fdisplay(syn_file, "WRITE_MODE=READ_FIRST,\\");
        $fdisplay(syn_file, "INITP_00=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_01=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_02=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_03=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_04=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_05=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_06=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_07=0000000000000000000000000000000000000000000000000000000000000000,\\");

        $fdisplay(syn_file_vhdl, "attribute xc_props of VROM : label is \"INIT=%s,\" &",  tohex8({8'h00, 1'b1, 1'b0, 1'b0, 1'b1, VBAND_VB, 11'd2047, VRGN_RST}));
        $fdisplay(syn_file_vhdl, "\"SRVAL=%s,\" &", tohex8({8'h00, 1'b1, 1'b0, 1'b0, 1'b1, VBAND_VB, 11'd2047, VRGN_RST}));
        $fdisplay(syn_file_vhdl, "\"WRITE_MODE=READ_FIRST,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_00=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_01=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_02=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_03=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_04=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_05=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_06=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_07=0000000000000000000000000000000000000000000000000000000000000000,\" &");

        // For each of the 64 INIT attributes, create a line with 256 bits 
        // encoded as hex and output to the various files.
        for (line = 0; line < 64; line = line + 1)
            begin
                line_num = line;
                $fwrite(sim_file,      "VROM.INIT_%s = 256'h", tohex2(line_num));
                $fwrite(sim_file_vhdl, "INIT_%s => X\"", tohex2(line_num));
                $fwrite(xst_file,      "//synthesis attribute INIT_%s of VROM is \"", tohex2(line_num));
                $fwrite(xst_file_vhdl, "attribute INIT_%s of VROM : label is \"", tohex2(line_num));
                $fwrite(syn_file,      "INIT_%s=", tohex2(line_num));
                $fwrite(syn_file_vhdl, "\"INIT_%s=", tohex2(line_num));
            
                for (word = 7; word >= 0; word = word - 1)
                    begin
                        word_num = word;
                        $fwrite(sim_file,      "%s", tohex8(vrom_value({line_num, word_num})));
                        $fwrite(sim_file_vhdl, "%s", tohex8(vrom_value({line_num, word_num})));
                        $fwrite(xst_file,      "%s", tohex8(vrom_value({line_num, word_num})));
                        $fwrite(xst_file_vhdl, "%s", tohex8(vrom_value({line_num, word_num})));
                        $fwrite(syn_file,      "%s", tohex8(vrom_value({line_num, word_num})));
                        $fwrite(syn_file_vhdl, "%s", tohex8(vrom_value({line_num, word_num})));
                    end
                if (line == 63)
                    begin
                        $fwrite(sim_file,      ";\n");
                        $fwrite(sim_file_vhdl, "\"\n");
                        $fwrite(xst_file,      "\"\n");
                        $fwrite(xst_file_vhdl, "\";\n");
                        $fwrite(syn_file,      "\"*/\n");
                        $fwrite(syn_file_vhdl, "\";\n");
                    end
                else
                    begin
                        $fwrite(sim_file,      ",\n");
                        $fwrite(sim_file_vhdl, "\",\n");
                        $fwrite(xst_file,      "\"\n");
                        $fwrite(xst_file_vhdl, "\";\n");
                        $fwrite(syn_file,      ",\\\n");
                        $fwrite(syn_file_vhdl, ",\" &\n");
                    end
            end

        // Now write any trailing lines to the files.
        $fdisplay(sim_file,"//synthesis translate_on");
        $fdisplay(sim_file_vhdl, ")");
        $fdisplay(sim_file_vhdl, "-- pragma translate_on");

        // Close the VROM files.
        $fclose(sim_file);
        $fclose(sim_file_vhdl);
        $fclose(xst_file);
        $fclose(xst_file_vhdl);
        $fclose(syn_file);
        $fclose(syn_file_vhdl);
    
        //
        // Create the CROM initialization files
        //
        
        // Open the six CROM output files.
        sim_file = $fopen("crom_sim.v");
        xst_file = $fopen("crom_xst.v");
        syn_file = $fopen("crom_syn.v");

        sim_file_vhdl = $fopen("crom_sim.vhd");
        xst_file_vhdl = $fopen("crom_xst.vhd");
        syn_file_vhdl = $fopen("crom_syn.vhd");

        // Write the header information for each file including the INIT_P
        // attributes. The parity bits are not used by the CROM.
        $fdisplay(sim_file, "// Simulation initialization code CROM");
        $fdisplay(sim_file, "// Created by multigenHD_romgen.v");
        $fdisplay(sim_file, "// Video format mapping:");
        $fdisplay(sim_file, "//   000 = %s", video_format_string(3'b000));
        $fdisplay(sim_file, "//   001 = %s", video_format_string(3'b001));
        $fdisplay(sim_file, "//   002 = %s", video_format_string(3'b010));
        $fdisplay(sim_file, "//   003 = %s", video_format_string(3'b011));
        $fdisplay(sim_file, "//   004 = %s", video_format_string(3'b100));
        $fdisplay(sim_file, "//   005 = %s", video_format_string(3'b101));
        $fdisplay(sim_file, "//   006 = %s", video_format_string(3'b110));
        $fdisplay(sim_file, "//   007 = %s", video_format_string(3'b111));
        $fdisplay(sim_file, "//synthesis translate_off");
        $fdisplay(sim_file, "defparam");
        $fdisplay(sim_file, "CROM.INIT = 36'h0%s,",  tohex8(crom_value({VBAND_VB, HRGN_RST, 1'b0})));
        $fdisplay(sim_file, "CROM.SRVAL = 36'h0%s,", tohex8(crom_value({VBAND_VB, HRGN_RST, 1'b0})));
        $fdisplay(sim_file, "CROM.WRITE_MODE = \"READ_FIRST\",");
        $fdisplay(sim_file, "CROM.INITP_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "CROM.INITP_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "CROM.INITP_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "CROM.INITP_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "CROM.INITP_04 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "CROM.INITP_05 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "CROM.INITP_06 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");
        $fdisplay(sim_file, "CROM.INITP_07 = 256'h0000000000000000000000000000000000000000000000000000000000000000,");

        $fdisplay(sim_file_vhdl, "-- Simulation initialization code CROM");
        $fdisplay(sim_file_vhdl, "-- Created by multigenHD_romgen.v");
        $fdisplay(sim_file_vhdl, "-- Video format mapping:");
        $fdisplay(sim_file_vhdl, "--   000 = %s", video_format_string(3'b000));
        $fdisplay(sim_file_vhdl, "--   001 = %s", video_format_string(3'b001));
        $fdisplay(sim_file_vhdl, "--   002 = %s", video_format_string(3'b010));
        $fdisplay(sim_file_vhdl, "--   003 = %s", video_format_string(3'b011));
        $fdisplay(sim_file_vhdl, "--   004 = %s", video_format_string(3'b100));
        $fdisplay(sim_file_vhdl, "--   005 = %s", video_format_string(3'b101));
        $fdisplay(sim_file_vhdl, "--   006 = %s", video_format_string(3'b110));
        $fdisplay(sim_file_vhdl, "--   007 = %s", video_format_string(3'b111));
        $fdisplay(sim_file_vhdl, "-- pragma translate_off");
        $fdisplay(sim_file_vhdl, "generic map (");
        $fdisplay(sim_file_vhdl, "INIT       => X\"%s\",", tohex8(crom_value({VBAND_VB, HRGN_RST, 1'b0})));
        $fdisplay(sim_file_vhdl, "SRVAL      => X\"%s\",", tohex8(crom_value({VBAND_VB, HRGN_RST, 1'b0})));
        $fdisplay(sim_file_vhdl, "WRITE_MODE => \"READ_FIRST\",");
        $fdisplay(sim_file_vhdl, "INITP_00 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_01 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_02 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_03 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_04 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_05 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_06 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
        $fdisplay(sim_file_vhdl, "INITP_07 => X\"0000000000000000000000000000000000000000000000000000000000000000\",");
    
        $fdisplay(xst_file, "// XST synthesis initialization code CROM");
        $fdisplay(xst_file, "// Created by multigenHD_romgen.v");
        $fdisplay(xst_file, "// Video format mapping:");
        $fdisplay(xst_file, "//   000 = %s", video_format_string(3'b000));
        $fdisplay(xst_file, "//   001 = %s", video_format_string(3'b001));
        $fdisplay(xst_file, "//   002 = %s", video_format_string(3'b010));
        $fdisplay(xst_file, "//   003 = %s", video_format_string(3'b011));
        $fdisplay(xst_file, "//   004 = %s", video_format_string(3'b100));
        $fdisplay(xst_file, "//   005 = %s", video_format_string(3'b101));
        $fdisplay(xst_file, "//   006 = %s", video_format_string(3'b110));
        $fdisplay(xst_file, "//   007 = %s", video_format_string(3'b111));
        $fdisplay(xst_file, "//synthesis attribute INIT of CROM is \"0%s\"",  tohex8(crom_value({VBAND_VB, HRGN_RST, 1'b0})));
        $fdisplay(xst_file, "//synthesis attribute SRVAL of CROM is \"0%s\"", tohex8(crom_value({VBAND_VB, HRGN_RST, 1'b0})));
        $fdisplay(xst_file, "//synthesis attribute WRITE_MODE of CROM is \"READ_FIRST\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_00 of CROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_01 of CROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_02 of CROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_03 of CROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_04 of CROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_05 of CROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_06 of CROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");
        $fdisplay(xst_file, "//synthesis attribute INITP_07 of CROM is \"0000000000000000000000000000000000000000000000000000000000000000\"");

        $fdisplay(xst_file_vhdl, "-- XST synthesis initialization code CROM");
        $fdisplay(xst_file_vhdl, "-- Created by multigenHD_romgen.v");
        $fdisplay(xst_file_vhdl, "-- Video format mapping:");
        $fdisplay(xst_file_vhdl, "--   000 = %s", video_format_string(3'b000));
        $fdisplay(xst_file_vhdl, "--   001 = %s", video_format_string(3'b001));
        $fdisplay(xst_file_vhdl, "--   002 = %s", video_format_string(3'b010));
        $fdisplay(xst_file_vhdl, "--   003 = %s", video_format_string(3'b011));
        $fdisplay(xst_file_vhdl, "--   004 = %s", video_format_string(3'b100));
        $fdisplay(xst_file_vhdl, "--   005 = %s", video_format_string(3'b101));
        $fdisplay(xst_file_vhdl, "--   006 = %s", video_format_string(3'b110));
        $fdisplay(xst_file_vhdl, "--   007 = %s", video_format_string(3'b111));
        $fdisplay(xst_file_vhdl, "attribute INIT of CROM : label is \"0%s\";",  tohex8(crom_value({VBAND_VB, HRGN_RST, 1'b0})));
        $fdisplay(xst_file_vhdl, "attribute SRVAL of CROM : label is \"0%s\";", tohex8(crom_value({VBAND_VB, HRGN_RST, 1'b0})));
        $fdisplay(xst_file_vhdl, "attribute WRITE_MODE of CROM : label is \"READ_FIRST\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_00 of CROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_01 of CROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_02 of CROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_03 of CROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_04 of CROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_05 of CROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_06 of CROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
        $fdisplay(xst_file_vhdl, "attribute INITP_07 of CROM : label is \"0000000000000000000000000000000000000000000000000000000000000000\";");
    
        $fdisplay(syn_file, "/* synthesis xc_props=\"INIT=0%s,\\",  tohex8(crom_value({VBAND_VB, HRGN_RST, 1'b0})));
        $fdisplay(syn_file, "SRVAL=0%s,\\", tohex8(crom_value({VBAND_VB, HRGN_RST, 1'b0})));
        $fdisplay(syn_file, "WRITE_MODE=READ_FIRST,\\");
        $fdisplay(syn_file, "INITP_00=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_01=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_02=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_03=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_04=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_05=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_06=0000000000000000000000000000000000000000000000000000000000000000,\\");
        $fdisplay(syn_file, "INITP_07=0000000000000000000000000000000000000000000000000000000000000000,\\");

        $fdisplay(syn_file_vhdl, "attribute xc_props of CROM : label is \"INIT=%s,\" &",  tohex8(crom_value({VBAND_VB, HRGN_RST, 1'b0})));
        $fdisplay(syn_file_vhdl, "\"SRVAL=%s,\" &", tohex8(crom_value({VBAND_VB, HRGN_RST, 1'b0})));
        $fdisplay(syn_file_vhdl, "\"WRITE_MODE=READ_FIRST,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_00=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_01=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_02=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_03=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_04=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_05=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_06=0000000000000000000000000000000000000000000000000000000000000000,\" &");
        $fdisplay(syn_file_vhdl, "\"INITP_07=0000000000000000000000000000000000000000000000000000000000000000,\" &");

        // For each of the 64 INIT attributes, create a line with 256 bits 
        // encoded as hex and output to the various files.
        for (line = 0; line < 64; line = line + 1)
            begin
                line_num = line;
                $fwrite(sim_file,      "CROM.INIT_%s = 256'h", tohex2(line_num));
                $fwrite(sim_file_vhdl, "INIT_%s => X\"", tohex2(line_num));
                $fwrite(xst_file,      "//synthesis attribute INIT_%s of CROM is \"", tohex2(line_num));
                $fwrite(xst_file_vhdl, "attribute INIT_%s of CROM : label is \"", tohex2(line_num));
                $fwrite(syn_file,      "INIT_%s=", tohex2(line_num));
                $fwrite(syn_file_vhdl, "\"INIT_%s=", tohex2(line_num));
        
                for (word = 7; word >= 0; word = word - 1)
                    begin
                        word_num = word;
                        $fwrite(sim_file,      "%s", tohex8(crom_value({line_num, word_num})));
                        $fwrite(sim_file_vhdl, "%s", tohex8(crom_value({line_num, word_num})));
                        $fwrite(xst_file,      "%s", tohex8(crom_value({line_num, word_num})));
                        $fwrite(xst_file_vhdl, "%s", tohex8(crom_value({line_num, word_num})));
                        $fwrite(syn_file,      "%s", tohex8(crom_value({line_num, word_num})));
                        $fwrite(syn_file_vhdl, "%s", tohex8(crom_value({line_num, word_num})));
                    end
                if (line == 63)
                    begin
                        $fwrite(sim_file,      ";\n");
                        $fwrite(sim_file_vhdl, "\"\n");
                        $fwrite(xst_file,      "\"\n");
                        $fwrite(xst_file_vhdl, "\";\n");
                        $fwrite(syn_file,      "\"*/\n");
                        $fwrite(syn_file_vhdl, "\";\n");
                    end
                else
                    begin
                        $fwrite(sim_file,      ",\n");
                        $fwrite(sim_file_vhdl, "\",\n");
                        $fwrite(xst_file,      "\"\n");
                        $fwrite(xst_file_vhdl, "\";\n");
                        $fwrite(syn_file,      ",\\\n");
                        $fwrite(syn_file_vhdl, ",\" &\n");
                    end
            end

        // Now write any trailing lines to the files.
        $fdisplay(sim_file,"//synthesis translate_on");
        $fdisplay(sim_file_vhdl, ")");
        $fdisplay(sim_file_vhdl, "-- pragma translate_on");

        // Close the HROM files.
        $fclose(sim_file);
        $fclose(sim_file_vhdl);
        $fclose(xst_file);
        $fclose(xst_file_vhdl);
        $fclose(syn_file);
        $fclose(syn_file_vhdl);

        $stop;
    end

//-----------------------------------------------------------------------------
// tohex1 function
//
// This function takes a 4-bit input vector and converts it to an ASCII
// character representing the HEX value of the input vector. All
// returned values are upper case to meet the requirements for naming the
// INIT_XX parameters.
//
function [7:0] tohex1;
input [3:0] d;

begin
    case(d)
        4'h0: tohex1 = "0";
        4'h1: tohex1 = "1";
        4'h2: tohex1 = "2";
        4'h3: tohex1 = "3";
        4'h4: tohex1 = "4";
        4'h5: tohex1 = "5";
        4'h6: tohex1 = "6";
        4'h7: tohex1 = "7";
        4'h8: tohex1 = "8";
        4'h9: tohex1 = "9";
        4'ha: tohex1 = "A";
        4'hb: tohex1 = "B";
        4'hc: tohex1 = "C";
        4'hd: tohex1 = "D";
        4'he: tohex1 = "E";
        4'hf: tohex1 = "F";
        default: tohex1 = "X";
    endcase 
end
endfunction


//-----------------------------------------------------------------------------
// tohex2 function
//
// This function takes an 8-bit input vector and converts it to two ASCII
// characters representing the hex value of the input vector. The results are
// returned in a 16-bit string.
//
function [15:0] tohex2;
input [7:0] d;
reg   [3:0] dl;
reg   [3:0] dm;
reg   [7:0] xl;
reg   [7:0] xm;

begin
    dl = d[3:0];
    dm = d[7:4];
    xl = tohex1(dl);
    xm = tohex1(dm);
    tohex2 = {xm, xl};
end
endfunction

//-----------------------------------------------------------------------------
// tohex8 function
//
// This function takes a 32-bit input vector and converts it to a string of
// eight ASCII characters representing the hex value of the input vector.
//
function [63:0] tohex8;
input [31:0] d;
reg   [7:0]  d0;
reg   [7:0]  d1;
reg   [7:0]  d2;
reg   [7:0]  d3;
reg   [15:0] x0;
reg   [15:0] x1;
reg   [15:0] x2;
reg   [15:0] x3;

begin
    d0 = d[7:0];
    d1 = d[15:8];
    d2 = d[23:16];
    d3 = d[31:24];
    x0 = tohex2(d0);
    x1 = tohex2(d1);
    x2 = tohex2(d2);
    x3 = tohex2(d3);
    tohex8 = {x3, x2, x1, x0};
end 
endfunction


//-----------------------------------------------------------------------------
// hrom_value function
//
// This function generates the horizontal ROM output value for any given input
// vector.
//
function [31:0] hrom_value;

input   [8:0]           d;

reg     [2:0]           std_code;
reg     [3:0]           std;
reg                     pattern;
reg     [HRGN_MSB:0]    region;
reg     [HRGN_MSB:0]    next_region;
reg     [H_EVNT_MSB:0]  next_event;
reg                     h_clr;
reg                     v_inc;
reg                     trs;
reg                     h;
reg                     usropt_rgn;
reg                     ceqpol_rgn;
reg                     eav2_rgn;
reg                     sav2_rgn;

begin
    std_code = d[8:6];
    std      = convert_std(std_code);
    pattern  = d[5];
    region   = d[4:0];

    if (region == HRGN_RST)
        begin
            next_region = HRGN_HBLANK;
            next_event = 11'd0000;
            h_clr = 1'b0;
            v_inc = 1'b1;
            trs = 1'b0;
            h = 1'b1;
            usropt_rgn = 1'b0;
            ceqpol_rgn = 1'b0;
            eav2_rgn = 1'b0;
            sav2_rgn = 1'b0;
        end
    else if (pattern == 0)      // color bars
        begin                               
            if (std != VSTD_296M)   // All 1920 active sample standards
                case(region)
                    HRGN_BAR0: begin
                                    next_region = HRGN_BAR1;
                                    next_event  = 11'd119;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR1:  begin
                                    next_region = HRGN_BAR2;
                                    next_event  = 11'd222;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b1;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR2:  begin
                                    next_region = HRGN_BAR3;
                                    next_event  = 11'd273;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR3:  begin
                                    next_region = HRGN_BAR4;
                                    next_event  = 11'd325;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR4:  begin
                                    next_region = HRGN_BAR5;
                                    next_event  = 11'd428;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR5:  begin
                                    next_region = HRGN_BAR6;
                                    next_event  = 11'd479;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR6:  begin
                                    next_region = HRGN_BAR7;
                                    next_event  = 11'd530;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR7:  begin
                                    next_region = HRGN_BAR8;
                                    next_event  = 11'd564;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR8:  begin
                                    next_region = HRGN_BAR9;
                                    next_event  = 11'd598;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR9:  begin
                                    next_region = HRGN_BAR10;
                                    next_event  = 11'd633;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR10: begin
                                    next_region = HRGN_BAR11;
                                    next_event  = 11'd667;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR11: begin
                                    next_region = HRGN_BAR12;
                                    next_event  = 11'd702;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR12: begin
                                    next_region = HRGN_BAR13;
                                    next_event  = 11'd736;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR13: begin
                                    next_region = HRGN_BAR14;
                                    next_event  = 11'd839;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR14: begin
                                    next_region = HRGN_INCV;
                                    next_event  = 11'd958;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_INCV:  begin
                                    next_region = HRGN_EAV1;
                                    next_event  = 11'd959;
                                    h_clr = 1'b0;
                                    v_inc = 1'b1;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_EAV1:  begin
                                    next_region = HRGN_EAV2_F0;
                                    next_event  = 11'd960;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_EAV2_F0: begin
                                    next_region = HRGN_LN;
                                    next_event  = 11'd961;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b1;
                                    sav2_rgn = 1'b0;
                                end
    // Note that the HROM contains control signals for controlling the generation
    // and insertion of the CRC and line number values into the video stream.
    // These signals are included in the ROMs but are currently unused by
    // multigenHD. The following two regions are used to create these control
    // signals.
                    HRGN_LN:    begin
                                    next_region = HRGN_CRC;
                                    next_event  = 11'd962;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_CRC:   begin
                                    next_region = HRGN_HBLANK;
                                    next_event  = 11'd963;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_HBLANK:begin
                                    next_region = HRGN_SAV1;
                                    case(std)
                                        VSTD_260M, VSTD_274M_30i, VSTD_274M_30p: // 30Hz formats
                                            next_event = 11'd1097;
                                        VSTD_274M_25i, VSTD_274M_25p:            // 1125 line 25Hz formats
                                            next_event = 11'd1317;
                                        VSTD_274M_24p, VSTD_274M_24sF:           // 24Hz formats
                                            next_event = 11'd1372;
                                        VSTD_295M:                               // 295M
                                            next_event = 11'd1185;
                                        default:
                                            next_event = 11'd0;
                                    endcase
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_SAV1:  begin
                                    next_region = HRGN_SAV2_F0;
                                    case(std)
                                        VSTD_260M, VSTD_274M_30i, VSTD_274M_30p:
                                            next_event = 11'd1098;
                                        VSTD_274M_25i, VSTD_274M_25p:
                                            next_event = 11'd1318;
                                        VSTD_274M_24p, VSTD_274M_24sF:
                                            next_event = 11'd1373;
                                        VSTD_295M:
                                            next_event = 11'd1186;
                                        default:
                                            next_event = 11'd0;
                                    endcase
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_SAV2_F0: begin
                                    next_region = HRGN_BAR0;
                                    case(std)
                                        VSTD_260M, VSTD_274M_30i, VSTD_274M_30p: // 30Hz formats
                                            next_event = 11'd1099;
                                        VSTD_274M_25i, VSTD_274M_25p:            // 1125 line 25Hz formats
                                            next_event = 11'd1319;
                                        VSTD_274M_24p, VSTD_274M_24sF:           // 24Hz formats
                                            next_event = 11'd1374;
                                        VSTD_295M:                               // 295M
                                            next_event = 11'd1187;
                                        default:
                                            next_event = 11'd0;
                                    endcase
                                    h_clr = 1'b1;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b1;
                                end
                    default:    begin
                                    next_region = HRGN_EAV1;
                                    next_event  = 11'd0;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                endcase

            else                            // 1280 active samples/line (SMPTE 296M 720p)
                case(region)
                    HRGN_BAR0: begin
                                    next_region = HRGN_BAR1;
                                    next_event  = 11'd79;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR1:  begin
                                    next_region = HRGN_BAR2;
                                    next_event  = 11'd149;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b1;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR2:  begin
                                    next_region = HRGN_BAR3;
                                    next_event  = 11'd182;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR3:  begin
                                    next_region = HRGN_BAR4;
                                    next_event  = 11'd217;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR4:  begin
                                    next_region = HRGN_BAR5;
                                    next_event  = 11'd285;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR5:  begin
                                    next_region = HRGN_BAR6;
                                    next_event  = 11'd319;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR6:  begin
                                    next_region = HRGN_BAR7;
                                    next_event  = 11'd353;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR7:  begin
                                    next_region = HRGN_BAR8;
                                    next_event  = 11'd376;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR8:  begin
                                    next_region = HRGN_BAR9;
                                    next_event  = 11'd399;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR9:  begin
                                    next_region = HRGN_BAR10;
                                    next_event  = 11'd421;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR10: begin
                                    next_region = HRGN_BAR11;
                                    next_event  = 11'd444;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR11: begin
                                    next_region = HRGN_BAR12;
                                    next_event  = 11'd466;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR12: begin
                                    next_region = HRGN_BAR13;
                                    next_event  = 11'd489;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR13: begin
                                    next_region = HRGN_BAR14;
                                    next_event  = 11'd559;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_BAR14: begin
                                    next_region = HRGN_INCV;
                                    next_event  = 11'd638;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_INCV:  begin
                                    next_region = HRGN_EAV1;
                                    next_event  = 11'd639;
                                    h_clr = 1'b0;
                                    v_inc = 1'b1;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_EAV1:  begin
                                    next_region = HRGN_EAV2_F0;
                                    next_event  = 11'd640;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_EAV2_F0: begin
                                    next_region = HRGN_LN;
                                    next_event  = 11'd641;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b1;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_LN:    begin
                                    next_region = HRGN_CRC;
                                    next_event  = 11'd642;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_CRC:   begin
                                    next_region = HRGN_HBLANK;
                                    next_event  = 11'd643;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_HBLANK:begin
                                    next_region = HRGN_SAV1;
                                    next_event = 11'd822;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_SAV1:  begin
                                    next_region = HRGN_SAV2_F0;
                                    next_event = 11'd823;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_SAV2_F0: begin
                                    next_region = HRGN_BAR0;
                                    next_event = 11'd824;
                                    h_clr = 1'b1;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b1;
                                end
                    default:    begin
                                    next_region = HRGN_EAV1;
                                    next_event  = 11'd0;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                endcase
        end
    else                    // pattern == 1, RP 198 checkfield
        begin
            if (std != VSTD_296M)   // All 1920 active sample standards
                case(region)
                    HRGN_CEQ_POL_0: begin
                                    next_region = HRGN_CEQ_PLL;
                                    next_event = 11'd1;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b1;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_CEQ_PLL: begin
                                    next_region = HRGN_INCV;
                                    next_event  = 11'd958;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_INCV:  begin
                                    next_region = HRGN_EAV1;
                                    next_event  = 11'd959;
                                    h_clr = 1'b0;
                                    v_inc = 1'b1;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_EAV1:  begin
                                    next_region = HRGN_EAV2_F0;
                                    next_event  = 11'd960;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_EAV2_F0: begin
                                    next_region = HRGN_LN;
                                    next_event  = 11'd961;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b1;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_LN:    begin
                                    next_region = HRGN_CRC;
                                    next_event  = 11'd962;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_CRC:   begin
                                    next_region = HRGN_HBLANK;
                                    next_event  = 11'd963;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_HBLANK:begin
                                    next_region = HRGN_SAV1;
                                    case(std)
                                        VSTD_260M, VSTD_274M_30i, VSTD_274M_30p: // 30Hz formats
                                            next_event = 11'd1097;
                                        VSTD_274M_25i, VSTD_274M_25p :           // 1125 line 25Hz formats
                                            next_event = 11'd1317;
                                        VSTD_274M_24p, VSTD_274M_24sF:           // 24Hz formats
                                            next_event = 11'd1372;
                                        VSTD_295M:                               // 295M
                                            next_event = 11'd1185;
                                        default:
                                            next_event = 11'd0;
                                    endcase
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_SAV1:  begin
                                    next_region = HRGN_SAV2_F0;
                                    case(std)
                                        VSTD_260M, VSTD_274M_30i, VSTD_274M_30p: // 30Hz formats
                                            next_event = 11'd1098;
                                        VSTD_274M_25i, VSTD_274M_25p :           // 1125 line 25Hz formats
                                            next_event = 11'd1318;
                                        VSTD_274M_24p, VSTD_274M_24sF:           // 24Hz formats
                                            next_event = 11'd1373;
                                        VSTD_295M:                               // 295M
                                            next_event = 11'd1186;
                                        default:
                                            next_event = 11'd0;
                                    endcase
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_SAV2_F0: begin
                                    next_region = HRGN_CEQ_POL_0;
                                    case(std)
                                        VSTD_260M, VSTD_274M_30i, VSTD_274M_30p: // 30Hz formats
                                            next_event = 11'd1099;
                                        VSTD_274M_25i, VSTD_274M_25p :           // 1125 line 25Hz formats
                                            next_event = 11'd1319;
                                        VSTD_274M_24p, VSTD_274M_24sF:           // 24Hz formats
                                            next_event = 11'd1374;
                                        VSTD_295M:                               // 295M
                                            next_event = 11'd1187;
                                        default:
                                            next_event = 11'd0;
                                    endcase
                                    h_clr = 1'b1;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b1;
                                end
                    default:    begin
                                    next_region = HRGN_EAV1;
                                    next_event  = 11'd0;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                endcase

            else                // SMPTE 296M standards with 1280 active samples
                case(region)
                    HRGN_CEQ_POL_0: begin
                                    next_region = HRGN_CEQ_PLL;
                                    next_event = 11'd1;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b1;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_CEQ_PLL: begin
                                    next_region = HRGN_INCV;
                                    next_event  = 11'd638;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_INCV:  begin
                                    next_region = HRGN_EAV1;
                                    next_event  = 11'd639;
                                    h_clr = 1'b0;
                                    v_inc = 1'b1;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_EAV1:  begin
                                    next_region = HRGN_EAV2_F0;
                                    next_event  = 11'd640;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_EAV2_F0: begin
                                    next_region = HRGN_LN;
                                    next_event  = 11'd641;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b1;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_LN:    begin
                                    next_region = HRGN_CRC;
                                    next_event  = 11'd642;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_CRC:   begin
                                    next_region = HRGN_HBLANK;
                                    next_event  = 11'd643;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_HBLANK:begin
                                    next_region = HRGN_SAV1;
                                    next_event  = 11'd822;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_SAV1:  begin
                                    next_region = HRGN_SAV2_F0;
                                    next_event  = 11'd823;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                    HRGN_SAV2_F0: begin
                                    next_region = HRGN_CEQ_POL_0;
                                    next_event  = 11'd824;
                                    h_clr = 1'b1;
                                    v_inc = 1'b0;
                                    trs = 1'b1;
                                    h = 1'b1;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b1;
                                end
                    default:    begin
                                    next_region = HRGN_EAV1;
                                    next_event  = 11'd0;
                                    h_clr = 1'b0;
                                    v_inc = 1'b0;
                                    trs = 1'b0;
                                    h = 1'b0;
                                    usropt_rgn = 1'b0;
                                    ceqpol_rgn = 1'b0;
                                    eav2_rgn = 1'b0;
                                    sav2_rgn = 1'b0;
                                end
                endcase
        end
                
    hrom_value = {3'b000, sav2_rgn, eav2_rgn, ceqpol_rgn, usropt_rgn, h, trs, v_inc, h_clr, region, next_event, next_region};
end

endfunction


//-----------------------------------------------------------------------------
// vrom_value function
//
// This function generates the vertical ROM output value for any given input
// vector.
//
function [31:0] vrom_value;

input   [8:0]           d;

reg     [2:0]           std_code;
reg     [3:0]           std;
reg                     pattern;
reg     [VRGN_MSB:0]    region;

reg     [VRGN_MSB:0]    next_region;
reg     [V_EVNT_MSB:0]  next_event;
reg     [VBAND_MSB:0]   band;
reg     [VBAND_MSB:0]   band_75;
reg                     v;
reg                     f;
reg                     frame;
reg                     v_clr;
reg                     first_line;
reg                     yramp_inc;

reg     [V_EVNT_MSB:0]  f0_vb_last;     // last line of v blanking interval, field 0
reg     [V_EVNT_MSB:0]  f0_first_act;   // first active line of field 0
reg     [V_EVNT_MSB:0]  f0_pat1_last;   // last line of pattern 1, field 0
reg     [V_EVNT_MSB:0]  f0_pat2_last;   // last line of pattern 2, field 0
reg     [V_EVNT_MSB:0]  f0_pat3_last;   // last line of pattern 3, field 0
reg     [V_EVNT_MSB:0]  f0_pat4_last;   // last line of pattern 4, field 0
reg     [V_EVNT_MSB:0]  f0_last_act;    // last active line of field 0
reg     [V_EVNT_MSB:0]  f0_last_line;   // last line of field 0
reg     [V_EVNT_MSB:0]  f1_vb_last;     // last line of v blanking interval, field 1
reg     [V_EVNT_MSB:0]  f1_first_act;   // first active line of field 1
reg     [V_EVNT_MSB:0]  f1_pat1_last;   // last line of pattern 1, field 1
reg     [V_EVNT_MSB:0]  f1_pat2_last;   // last line of pattern 2, field 1
reg     [V_EVNT_MSB:0]  f1_pat3_last;   // last line of pattern 3, field 1
reg     [V_EVNT_MSB:0]  f1_pat4_last;   // last line of pattern 4, field 1
reg     [V_EVNT_MSB:0]  f1_last_act;    // last active line of field 1
reg     [V_EVNT_MSB:0]  fm_last_line;   // last line of frame

reg                     progressive;    // 1 for progressive standards

begin
    std_code = d[8:6];
    std      = convert_std(std_code);
    pattern  = d[5];
    region   = d[4:0];
    
    progressive = is_progressive(std);
    yramp_inc = is_1280active(std);

    // default the various timing parameters
    f0_vb_last      = 0;
    f0_first_act    = 0;
    f0_pat1_last    = 0;
    f0_pat2_last    = 0;
    f0_pat3_last    = 0;
    f0_pat4_last    = 0;
    f0_last_act     = 0;
    f0_last_line    = 0;
    f1_vb_last      = 0;
    f1_first_act    = 0;
    f1_pat1_last    = 0;
    f1_pat2_last    = 0;
    f1_pat3_last    = 0;
    f1_pat4_last    = 0;
    f1_last_act     = 0;
    fm_last_line    = 0;

    // setup the various timing parameters based on std and pattern
    if (std == VSTD_260M)
        begin
            f0_vb_last      = 11'd40;
            f0_first_act    = f0_vb_last + 11'd1;
            f0_last_act     = 11'd557;
            f0_last_line    = 11'd563;
            f1_vb_last      = 11'd602;
            f1_first_act    = f1_vb_last + 11'd1;
            f1_last_act     = 11'd1120;
            fm_last_line    = 11'd1125;
            if (pattern == 1'b0)
                begin
                    f0_pat1_last = f0_vb_last   + 11'd302;
                    f0_pat2_last = f0_pat1_last + 11'd43;
                    f0_pat3_last = f0_pat2_last + 11'd43;
                    f0_pat4_last = f0_pat3_last + 11'd129;
                    f1_pat1_last = f1_vb_last   + 11'd302;
                    f1_pat2_last = f1_pat1_last + 11'd43;
                    f1_pat3_last = f1_pat2_last + 11'd43;
                    f1_pat4_last = f1_pat3_last + 11'd130;
                end
            else
                begin
                    f0_pat1_last = 11'd298;
                    f0_pat2_last = f0_last_act;
                    f1_pat1_last = 11'd861;
                    f1_pat2_last = f1_last_act;
                end
        end
    else if (std == VSTD_295M)
        begin
            f0_vb_last      = 11'd80;
            f0_first_act    = f0_vb_last + 11'd1;
            f0_last_act     = 11'd620;
            f0_last_line    = 11'd625;
            f1_vb_last      = 11'd705;
            f1_first_act    = f1_vb_last + 11'd1;
            f1_last_act     = 11'd1245;
            fm_last_line    = 11'd1250;
            if (pattern == 1'b0)
                begin
                    f0_pat1_last = f0_vb_last   + 11'd315;
                    f0_pat2_last = f0_pat1_last + 11'd45;
                    f0_pat3_last = f0_pat2_last + 11'd45;
                    f0_pat4_last = f0_pat3_last + 11'd135;
                    f1_pat1_last = f1_vb_last   + 11'd315;
                    f1_pat2_last = f1_pat1_last + 11'd45;
                    f1_pat3_last = f1_pat2_last + 11'd45;
                    f1_pat4_last = f1_pat3_last + 11'd135;
                end
            else
                begin
                    f0_pat1_last = 11'd350;
                    f0_pat2_last = f0_last_act;
                    f1_pat1_last = 11'd975;
                    f1_pat2_last = f1_last_act;
                end
        end
    else if (std == VSTD_274M_30i || std == VSTD_274M_25i || std == VSTD_274M_24sF)
        begin
            f0_vb_last      = 11'd20;
            f0_first_act    = f0_vb_last + 11'd1;
            f0_last_act     = 11'd560;
            f0_last_line    = 11'd563;
            f1_vb_last      = 11'd583;
            f1_first_act    = f1_vb_last + 11'd1;
            f1_last_act     = 11'd1123;
            fm_last_line    = 11'd1125;
            if (pattern == 1'b0)
                begin
                    f0_pat1_last = f0_vb_last   + 11'd315;
                    f0_pat2_last = f0_pat1_last + 11'd45;
                    f0_pat3_last = f0_pat2_last + 11'd45;
                    f0_pat4_last = f0_pat3_last + 11'd135;
                    f1_pat1_last = f1_vb_last   + 11'd315;
                    f1_pat2_last = f1_pat1_last + 11'd45;
                    f1_pat3_last = f1_pat2_last + 11'd45;
                    f1_pat4_last = f1_pat3_last + 11'd135;
                end
            else
                begin
                    f0_pat1_last = 11'd290;
                    f0_pat2_last = f0_last_act;
                    f1_pat1_last = 11'd853;
                    f1_pat2_last = f1_last_act;
                end
        end
    else if (std == VSTD_274M_30p || std == VSTD_274M_25p || std == VSTD_274M_24p)
        begin
            f0_vb_last      = 11'd41;
            f0_first_act    = f0_vb_last + 11'd1;
            f0_last_act     = 11'd1121;
            fm_last_line    = 11'd1125;
            f0_last_line    = 11'd1125;
            if (pattern == 1'b0)
                begin
                    f0_pat1_last = f0_vb_last   + 11'd630;
                    f0_pat2_last = f0_pat1_last + 11'd90;
                    f0_pat3_last = f0_pat2_last + 11'd90;
                    f0_pat4_last = f0_pat3_last + 11'd270;
                end
            else
                begin
                    f0_pat1_last = 11'd581;
                    f0_pat2_last = f0_last_act;
                end
        end
    else /*if (std == VSTD_296M)*/
        begin
            f0_vb_last      = 11'd25;
            f0_first_act    = f0_vb_last + 11'd1;
            f0_last_act     = 11'd745;
            fm_last_line    = 11'd750;
            f0_last_line    = 11'd750;
            if (pattern == 1'b0)
                begin
                    f0_pat1_last = f0_vb_last   + 11'd420;
                    f0_pat2_last = f0_pat1_last + 11'd60;
                    f0_pat3_last = f0_pat2_last + 11'd60;
                    f0_pat4_last = f0_pat3_last + 11'd180;
                end
            else
                begin
                    f0_pat1_last = 11'd385;
                    f0_pat2_last = f0_last_act;
                end
        end
 
    // Implement the sequencer based on the above timing parameters
    case(region)
        VRGN_RST:
                            begin
                                next_region = VRGN_FM0_F0_VB_0;
                                next_event = 11'd1;
                                band = VBAND_VB;
                                band_75 = VBAND_VB;
                                v = 1'b1;
                                f = 1'b0;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        // First frame
        VRGN_FM0_F0_VB_0:   begin
                                next_region = VRGN_FM0_F0_1ST_ACT;
                                next_event = f0_vb_last;
                                band = VBAND_VB;
                                band_75 = VBAND_VB;
                                v = 1'b1;
                                f = 1'b0;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM0_F0_1ST_ACT:begin
                                next_region = VRGN_FM0_F0_PAT1;
                                next_event = f0_first_act;
                                band = pattern ? VBAND_CEQ : VBAND_PAT1;
                                band_75 = pattern ? VBAND_CEQ : VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b0;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b1;
                            end
        VRGN_FM0_F0_PAT1:   begin
                                next_region = VRGN_FM0_F0_PAT2;
                                next_event = f0_pat1_last;
                                band = pattern ? VBAND_CEQ: VBAND_PAT1;
                                band_75 = pattern ? VBAND_CEQ: VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b0;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM0_F0_PAT2:   begin
                                next_region = pattern ? VRGN_FM0_F0_VB_1 : VRGN_FM0_F0_PAT3;
                                next_event = f0_pat2_last;
                                band = pattern ? VBAND_PLL: VBAND_PAT2;
                                band_75 = pattern ? VBAND_PLL: VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b0;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM0_F0_PAT3:   begin
                                next_region = VRGN_FM0_F0_PAT4;
                                next_event = f0_pat3_last;
                                band = VBAND_PAT3;
                                band_75 = VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b0;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM0_F0_PAT4:   begin
                                next_region = VRGN_FM0_F0_VB_1;
                                next_event = f0_pat4_last;
                                band = VBAND_PAT4;
                                band_75 = VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b0;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM0_F0_VB_1:   begin
                                next_region = progressive ? VRGN_FM0_CLRV : VRGN_FM0_F1_VB_0;
                                next_event = progressive ? fm_last_line - 11'd1 : f0_last_line;
                                //next_event = f0_last_line;
                                band = VBAND_VB;
                                band_75 = VBAND_VB;
                                v = 1'b1;
                                f = 1'b0;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM0_F1_VB_0:   begin
                                next_region = VRGN_FM0_F1_PAT1;
                                next_event = f1_vb_last;
                                band = VBAND_VB;
                                band_75 = VBAND_VB;
                                v = 1'b1;
                                f = 1'b1;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM0_F1_PAT1:   begin
                                next_region = VRGN_FM0_F1_PAT2;
                                next_event = f1_pat1_last;
                                band = pattern ? VBAND_CEQ: VBAND_PAT1;
                                band_75 = pattern ? VBAND_CEQ: VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b1;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM0_F1_PAT2:   begin
                                next_region = pattern ? VRGN_FM0_F1_VB_1 : VRGN_FM0_F1_PAT3;
                                next_event = f1_pat2_last;
                                band = pattern ? VBAND_PLL: VBAND_PAT2;
                                band_75 = pattern ? VBAND_PLL: VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b1;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM0_F1_PAT3:   begin
                                next_region = VRGN_FM0_F1_PAT4;
                                next_event = f1_pat3_last;
                                band = VBAND_PAT3;
                                band_75 = VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b1;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM0_F1_PAT4:   begin
                                next_region = VRGN_FM0_F1_VB_1;
                                next_event = f1_pat4_last;
                                band = VBAND_PAT4;
                                band_75 = VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b1;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM0_F1_VB_1:   begin
                                next_region = VRGN_FM0_CLRV;
                                next_event = fm_last_line - 11'd1;
                                band = VBAND_VB;
                                band_75 = VBAND_VB;
                                v = 1'b1;
                                f = 1'b1;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM0_CLRV:      begin
                                next_region = VRGN_FM1_F0_VB_0;
                                next_event = fm_last_line;
                                band = VBAND_VB;
                                band_75 = VBAND_VB;
                                v = 1'b1;
                                f = ~progressive;
                                frame = 1'b0;
                                v_clr = 1'b1;
                                first_line = 1'b0;
                            end

        // Second frame
        VRGN_FM1_F0_VB_0:   begin
                                next_region = VRGN_FM1_F0_PAT1;
                                next_event = f0_vb_last;
                                band = VBAND_VB;
                                band_75 = VBAND_VB;
                                v = 1'b1;
                                f = 1'b0;
                                frame = 1'b1;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM1_F0_PAT1:   begin
                                next_region = VRGN_FM1_F0_PAT2;
                                next_event = f0_pat1_last;
                                band = pattern ? VBAND_CEQ: VBAND_PAT1;
                                band_75 = pattern ? VBAND_CEQ: VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b0;
                                frame = 1'b1;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM1_F0_PAT2:   begin
                                next_region = pattern ? VRGN_FM1_F0_VB_1 : VRGN_FM1_F0_PAT3;
                                next_event = f0_pat2_last;
                                band = pattern ? VBAND_PLL: VBAND_PAT2;
                                band_75 = pattern ? VBAND_PLL: VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b0;
                                frame = 1'b1;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM1_F0_PAT3:   begin
                                next_region = VRGN_FM1_F0_PAT4;
                                next_event = f0_pat3_last;
                                band = VBAND_PAT3;
                                band_75 = VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b0;
                                frame = 1'b1;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM1_F0_PAT4:   begin
                                next_region = VRGN_FM1_F0_VB_1;
                                next_event = f0_pat4_last;
                                band = VBAND_PAT4;
                                band_75 = VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b0;
                                frame = 1'b1;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM1_F0_VB_1:   begin
                                next_region = progressive ? VRGN_FM1_CLRV : VRGN_FM1_F1_VB_0;
                                next_event = progressive ? fm_last_line - 11'd1 : f0_last_line;
                                //next_event = f0_last_line;
                                band = VBAND_VB;
                                band_75 = VBAND_VB;
                                v = 1'b1;
                                f = 1'b0;
                                frame = 1'b1;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM1_F1_VB_0:   begin
                                next_region = VRGN_FM1_F1_PAT1;
                                next_event = f1_vb_last;
                                band = VBAND_VB;
                                band_75 = VBAND_VB;
                                v = 1'b1;
                                f = 1'b1;
                                frame = 1'b1;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM1_F1_PAT1:   begin
                                next_region = VRGN_FM1_F1_PAT2;
                                next_event = f1_pat1_last;
                                band = pattern ? VBAND_CEQ: VBAND_PAT1;
                                band_75 = pattern ? VBAND_CEQ: VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b1;
                                frame = 1'b1;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM1_F1_PAT2:   begin
                                next_region = pattern ? VRGN_FM1_F1_VB_1 : VRGN_FM1_F1_PAT3;
                                next_event = f1_pat2_last;
                                band = pattern ? VBAND_PLL: VBAND_PAT2;
                                band_75 = pattern ? VBAND_PLL: VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b1;
                                frame = 1'b1;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM1_F1_PAT3:   begin
                                next_region = VRGN_FM1_F1_PAT4;
                                next_event = f1_pat3_last;
                                band = VBAND_PAT3;
                                band_75 = VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b1;
                                frame = 1'b1;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM1_F1_PAT4:   begin
                                next_region = VRGN_FM1_F1_VB_1;
                                next_event = f1_pat4_last;
                                band = VBAND_PAT4;
                                band_75 = VBAND_PAT1;
                                v = 1'b0;
                                f = 1'b1;
                                frame = 1'b1;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM1_F1_VB_1:   begin
                                next_region = VRGN_FM1_CLRV;
                                next_event = fm_last_line - 11'd1;
                                band = VBAND_VB;
                                band_75 = VBAND_VB;
                                v = 1'b1;
                                f = 1'b1;
                                frame = 1'b1;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
        VRGN_FM1_CLRV:      begin
                                next_region = VRGN_FM0_F0_VB_0;
                                next_event = fm_last_line;
                                band = VBAND_VB;
                                band_75 = VBAND_VB;
                                v = 1'b1;
                                f = ~progressive;
                                frame = 1'b1;
                                v_clr = 1'b1;
                                first_line = 1'b0;
                            end
        default:            begin
                                next_region = VRGN_FM0_F0_VB_0;
                                next_event = fm_last_line;
                                band = VBAND_VB;
                                band_75 = VBAND_VB;
                                v = 1'b1;
                                f = 1'b0;
                                frame = 1'b0;
                                v_clr = 1'b0;
                                first_line = 1'b0;
                            end
    endcase
          
    vrom_value = {4'b0000, band_75, yramp_inc, first_line, v_clr, frame, f, v, band, next_event, next_region};
end
endfunction


//-----------------------------------------------------------------------------
// crom_value function
//
// This function generates the color ROM output value for any given input
// vector.
//
function [31:0] crom_value;

input   [8:0]           d;

reg     [2:0]           v_band;
reg     [HRGN_MSB:0]    h_region;
reg                     hcnt_lsb;

reg     [VID_MSB:0]     y;
reg     [VID_MSB:0]     c;
reg                     y_ramp_en;
reg                     y_ramp_reload;

begin
    v_band      = d[8:6];
    h_region    = d[5:1];
    hcnt_lsb    = d[0];

    y_ramp_en = 1'b0;
    y_ramp_reload = 1'b0;
    y = Y_BLANK;
    c = CB_BLANK;

    if (v_band == VBAND_VB)
        case(h_region)
            HRGN_EAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_EAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b011) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b011) : 10'h000;
                        end
            HRGN_EAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b111) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b111) : 10'h000;
                        end
            HRGN_SAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_SAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b010) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b010) : 10'h000;
                        end
            HRGN_SAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b110) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b110) : 10'h000;
                        end
        endcase

    else if (v_band == VBAND_PAT1)
        case(h_region)
            HRGN_BAR0, HRGN_BAR14, HRGN_INCV:   
                        begin
                            y = Y_GRAY40;
                            c = hcnt_lsb ? CR_GRAY40 : CB_GRAY40;
                        end
            HRGN_BAR1, HRGN_USROPT1, HRGN_USROPT2, HRGN_USROPT3:    
                        begin
                            y = Y_WHITE75;
                            c = hcnt_lsb ? CR_WHITE75 : CB_WHITE75;
                        end
            HRGN_BAR2, HRGN_BAR3:   
                        begin
                            y = Y_YELLOW75;
                            c = hcnt_lsb ? CR_YELLOW75 : CB_YELLOW75;
                        end
            HRGN_BAR4:  begin
                            y = Y_CYAN75;
                            c = hcnt_lsb ? CR_CYAN75 : CB_CYAN75;
                        end
            HRGN_BAR5, HRGN_BAR6:   
                        begin
                            y = Y_GREEN75;
                            c = hcnt_lsb ? CR_GREEN75 : CB_GREEN75;
                        end
            HRGN_BAR7, HRGN_BAR8, HRGN_BAR9:
                        begin
                            y = Y_MAGENTA75;
                            c = hcnt_lsb ? CR_MAGENTA75 : CB_MAGENTA75;
                        end
            HRGN_BAR10, HRGN_BAR11, HRGN_BAR12:
                        begin
                            y = Y_RED75;
                            c = hcnt_lsb ? CR_RED75 : CB_RED75;
                        end
            HRGN_BAR13: begin
                            y = Y_BLUE75;
                            c = hcnt_lsb ? CR_BLUE75 : CB_BLUE75;
                        end
            HRGN_EAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_EAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b001) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b001) : 10'h000;
                        end
            HRGN_EAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b101) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b101) : 10'h000;
                        end
            HRGN_SAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_SAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b000) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b000) : 10'h000;
                        end
            HRGN_SAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b100) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b100) : 10'h000;
                        end
        endcase

    else if (v_band == VBAND_PAT2)
        case(h_region)
            HRGN_BAR0:  begin
                            y = Y_CYAN100;
                            c = hcnt_lsb ? CR_CYAN100 : CB_CYAN100;
                        end
            HRGN_BAR1:  begin
                            y = Y_WHITE75;
                            c = hcnt_lsb ? CR_WHITE75 : CB_WHITE75;
                        end
            HRGN_USROPT1: begin
                            y = Y_WHITE100;
                            c = hcnt_lsb ? CR_WHITE100 : CB_WHITE100;
                        end
            HRGN_USROPT2: begin
                            y = Y_PLUS_I;
                            c = hcnt_lsb ? CR_PLUS_I : CB_PLUS_I;
                        end
            HRGN_USROPT3: begin
                            y = Y_MINUS_I;
                            c = hcnt_lsb ? CR_MINUS_I : CB_MINUS_I;
                        end
            HRGN_BAR2, HRGN_BAR3, HRGN_BAR4, HRGN_BAR5, HRGN_BAR6, HRGN_BAR7,
            HRGN_BAR8, HRGN_BAR9, HRGN_BAR10, HRGN_BAR11, HRGN_BAR12, HRGN_BAR13:   
                        begin
                            y = Y_WHITE75;
                            c = hcnt_lsb ? CR_WHITE75 : CB_WHITE75;
                        end
            HRGN_BAR14, HRGN_INCV:
                        begin
                            y = Y_BLUE100;
                            c = hcnt_lsb ? CR_BLUE100 : CB_BLUE100;
                        end
            HRGN_EAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_EAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b001) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b001) : 10'h000;
                        end
            HRGN_EAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b101) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b101) : 10'h000;
                        end
            HRGN_SAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_SAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b000) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b000) : 10'h000;
                        end
            HRGN_SAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b100) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b100) : 10'h000;
                        end
        endcase

    else if (v_band == VBAND_PAT3)
        case(h_region)
            HRGN_BAR0:  begin
                            y = Y_YELLOW100;
                            c = hcnt_lsb ? CR_YELLOW100 : CB_YELLOW100;
                        end
            HRGN_BAR1, HRGN_USROPT1, HRGN_USROPT2:  
                        begin
                            y_ramp_reload = 1'b1;
                            y = Y_BLK0;
                            c = hcnt_lsb ? CR_BLK0 : CB_BLK0;
                        end
            HRGN_USROPT3: begin
                            y_ramp_reload = 1'b1;
                            y = Y_PLUS_Q;
                            c = hcnt_lsb ? CR_PLUS_Q : CB_PLUS_Q;
                        end
            HRGN_BAR2, HRGN_BAR3, HRGN_BAR4, HRGN_BAR5, HRGN_BAR6, HRGN_BAR7,
            HRGN_BAR8, HRGN_BAR9, HRGN_BAR10, HRGN_BAR11, HRGN_BAR12, HRGN_BAR13:   
                        begin
                            y_ramp_en = 1'b1;
                            y = Y_WHITE100;     // overwritten by ramp logic
                            c = hcnt_lsb ? CR_WHITE100 : CB_WHITE100;
                        end
            HRGN_BAR14, HRGN_INCV:
                        begin
                            y = Y_RED100;
                            c = hcnt_lsb ? CR_RED100 : CB_RED100;
                        end
            HRGN_EAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_EAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b001) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b001) : 10'h000;
                        end
            HRGN_EAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b101) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b101) : 10'h000;
                        end
            HRGN_SAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_SAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b000) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b000) : 10'h000;
                        end
            HRGN_SAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b100) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b100) : 10'h000;
                        end
        endcase

    else if (v_band == VBAND_PAT4)
        case(h_region)
            HRGN_BAR0, HRGN_BAR14, HRGN_INCV:
                        begin
                            y = Y_GRAY15;
                            c = hcnt_lsb ? CR_GRAY15 : CB_GRAY15;
                        end
            HRGN_BAR1, HRGN_USROPT1, HRGN_USROPT2, HRGN_USROPT3, HRGN_BAR2,
            HRGN_BAR6, HRGN_BAR7, HRGN_BAR9, HRGN_BAR11, HRGN_BAR13:    
                        begin
                            y = Y_BLK0;
                            c = hcnt_lsb ? CR_BLK0 : CB_BLK0;
                        end
            HRGN_BAR3, HRGN_BAR4, HRGN_BAR5:    
                        begin
                            y = Y_WHITE100;
                            c = hcnt_lsb ? CR_WHITE100 : CB_WHITE100;
                        end
            HRGN_BAR8:  begin
                            y = Y_BLK_M2;
                            c = hcnt_lsb ? CR_BLK_M2 : CB_BLK_M2;
                        end
            HRGN_BAR10:
                        begin
                            y = Y_BLK_P2;
                            c = hcnt_lsb ? CR_BLK_P2 : CB_BLK_P2;
                        end
            HRGN_BAR12: begin
                            y = Y_BLK_P4;
                            c = hcnt_lsb ? CR_BLK_P4 : CB_BLK_P4;
                        end
            HRGN_EAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_EAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b001) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b001) : 10'h000;
                        end
            HRGN_EAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b101) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b101) : 10'h000;
                        end
            HRGN_SAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_SAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b000) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b000) : 10'h000;
                        end
            HRGN_SAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b100) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b100) : 10'h000;
                        end
        endcase

    else if (v_band == VBAND_CEQ)
        case(h_region)
            HRGN_CEQ_PLL, HRGN_CEQ_POL_0, HRGN_INCV: begin
                            y = Y_CEQ;
                            c = C_CEQ;
                        end
            HRGN_CEQ_POL_1: begin
                            y = Y_CEQ_INV;
                            c = C_CEQ;
                        end
            HRGN_EAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_EAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b001) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b001) : 10'h000;
                        end
            HRGN_EAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b101) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b101) : 10'h000;
                        end
            HRGN_SAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_SAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b000) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b000) : 10'h000;
                        end
            HRGN_SAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b100) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b100) : 10'h000;
                        end
        endcase


    else if (v_band == VBAND_PLL)
        case(h_region)
            HRGN_CEQ_PLL, HRGN_CEQ_POL_0, HRGN_CEQ_POL_1, HRGN_INCV: 
                        begin
                            y = Y_PLL;
                            c = C_PLL;
                        end
            HRGN_EAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_EAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b001) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b001) : 10'h000;
                        end
            HRGN_EAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b101) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b101) : 10'h000;
                        end
            HRGN_SAV1:  begin
                            y = hcnt_lsb ? 10'h000 : 10'h3ff;
                            c = hcnt_lsb ? 10'h000 : 10'h3ff;
                        end
            HRGN_SAV2_F0: begin
                            y = hcnt_lsb ? calc_trs(3'b000) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b000) : 10'h000;
                        end
            HRGN_SAV2_F1: begin
                            y = hcnt_lsb ? calc_trs(3'b100) : 10'h000;
                            c = hcnt_lsb ? calc_trs(3'b100) : 10'h000;
                        end
        endcase

    crom_value = {8'h00, c, 2'b00, y, y_ramp_en, y_ramp_reload};
end
endfunction

//-----------------------------------------------------------------------------
// calc_trs function
//
// This function computes the TRS XYZ word based on the F, V, and H bits.
//
function [VID_MSB:0] calc_trs;
input [2:0] flags;

reg     f;
reg     v;
reg     h;
reg     p3;
reg     p2;
reg     p1;
reg     p0;
begin
    f = flags[2];
    v = flags[1];
    h = flags[0];

    p3 = v ^ h;
    p2 = f ^ h;
    p1 = f ^ v;
    p0 = f ^ v ^ h;

    calc_trs = {1'b1, f, v, h, p3, p2, p1, p0, 2'b00};

end
endfunction


//-----------------------------------------------------------------------------
// video_format_string function
//
// This function converts the 3-bit standard select code into a string
// describing the video format mapped to that select code.
//
function [43*8:0] video_format_string;
input [2:0] std_select;

reg [3:0] std;

begin
    std = convert_std(std_select);

    case(std)
        4'b0000: video_format_string = "SMPTE 260M - 1035i  30Hz & 29.97 Hz        ";
        4'b0001: video_format_string = "SMPTE 295M - 1080i  25Hz (1250 lines/frame)";
        4'b0010: video_format_string = "SMPTE 274M - 1080i  30Hz & 29.97 Hz        ";
        4'b0011: video_format_string = "SMPTE 274M - 1080i  25Hz                   ";
        4'b0100: video_format_string = "SMPTE 274M - 1080p  30Hz & 29.97Hz         ";
        4'b0101: video_format_string = "SMPTE 274M - 1080p  25Hz                   ";
        4'b0110: video_format_string = "SMPTE 274M - 1080p  24Hz & 23.98Hz         ";
        4'b0111: video_format_string = "SMPTE 296M - 720p   60Hz & 59.94Hz         ";
        4'b1000: video_format_string = "SMPTE 274M - 1080sF 24Hz & 23.98Hz         ";
        default: video_format_string = "Undefined                                  ";
    endcase
end 
endfunction 


//-----------------------------------------------------------------------------
// is_progressive function
//
// This function returns a value of 1'b1 if the 4-bit input code corresponds
// to a progressive video format, otherwise it returns 1'b0.
//
function is_progressive;
input [3:0] std;

begin
    case(std)
        VSTD_274M_30p:  is_progressive = 1'b1;
        VSTD_274M_25p:  is_progressive = 1'b1;
        VSTD_274M_24p:  is_progressive = 1'b1;
        VSTD_296M:      is_progressive = 1'b1;
        default:        is_progressive = 1'b0;
    endcase 
end
endfunction


//-----------------------------------------------------------------------------
// is_1280active function
//
// This function returns a value of 1'b1 if the 4-bit input code corresponds
// to a video format with 1280 active samples per lines, otherwise it returns
// 1'b0.
//
function is_1280active;
input [3:0] std;

begin
    if (std == VSTD_296M)
        is_1280active = 1'b1;
    else
        is_1280active = 1'b0;
end         
endfunction

//-----------------------------------------------------------------------------
// convert_std function
//
// This function maps the 3-bit ROM address video standard select to a 4-bit
// video format code that is used throughout this hrom_value and vrom_value
// functions.
//
// This function effectively selects eight video formats for inclusion in the
// ROMs. By changing the case statement in this function, different video
// formats can be included or the mapping from the ROM std inputs to the
// various formats can be altered.
//
function [3:0] convert_std;
input [2:0] std_code;

begin
    case(std_code)
//      3'b000:  convert_std = VSTD_260M;           // This format is not generated by default
        3'b000:  convert_std = VSTD_295M;           
        3'b001:  convert_std = VSTD_274M_24sF;
        3'b010:  convert_std = VSTD_274M_30i;
        3'b011:  convert_std = VSTD_274M_25i;
        3'b100:  convert_std = VSTD_274M_30p;
        3'b101:  convert_std = VSTD_274M_25p;
        3'b110:  convert_std = VSTD_274M_24p;
        3'b111:  convert_std = VSTD_296M;
    endcase
end 
endfunction

endmodule

